import os
import pickle
from func import *
from idc import *
import idaapi

# 定义常量
DATA_DIR = "D:\\bishe\\dataset"
INFECTED_DIR = os.path.join(DATA_DIR, "infected")
BENIGN_DIR = os.path.join(DATA_DIR, "benign")
CFG_EXTENSION = ".ida"
GDL_EXTENSION = ".dot"
ASM_EXTENSION = ".asm"


def preprocess(binary_name, workflow):
    cfg_path = os.path.join(
        INFECTED_DIR if workflow != "-1" else BENIGN_DIR,
        f"{binary_name}{CFG_EXTENSION}"
    )
    gdl_path = os.path.join(
        INFECTED_DIR if workflow != "-1" else BENIGN_DIR,
        f"{binary_name}{GDL_EXTENSION}"
    )
    asm_path = os.path.join(
        INFECTED_DIR if workflow != "-1" else BENIGN_DIR,
        f"{binary_name}{ASM_EXTENSION}"
    )

    if os.path.exists(cfg_path):
        idc.Exit(0)
    else:
        analysis_flags = idc.GetShortPrm(idc.INF_AF2)
        analysis_flags &= ~ida_ida.AF_IMMOFF
        idc.SetShortPrm(idc.INF_AF2, analysis_flags)

        idaapi.autoWait()

        # 生成CFG
        generate_cfg(binary_name, cfg_path)

        # 生成GDL
        generate_gdl(gdl_path)

        # 生成ASM
        generate_asm(asm_path)

        # 关闭IDA Pro
        idc.Exit(0)


def generate_cfg(binary_name, cfg_path):
    cfgs = get_func_cfgs_c(FirstSeg())
    with open(cfg_path, 'wb') as cfg_file:
        pickle.dump(cfgs, cfg_file)


def generate_gdl(gdl_path):
    idc.GenCallGdl(gdl_path, 'Call Gdl', idaapi.CHART_GEN_DOT)


def generate_asm(asm_path):
    idc.GenerateFile(idc.OFILE_ASM, asm_path, 0, idc.BADADDR, 0)


# 主函数
def main():
    binary_name = idc.GetInputFile()
    try:
        workflow = idc.ARGV[1]
    except IndexError:
        print("Workflow argument not provided.")
        return
    preprocess(binary_name, workflow)


# 如果是作为IDA Pro的脚本运行，调用主函数
if __name__ == "__main__":
    main()
